package com.istnetworks.hivesdk.domain.usecases

import android.util.Log
import androidx.core.util.Pair
import androidx.lifecycle.MutableLiveData
import com.istnetworks.hivesdk.data.models.QuestionResponses
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.presentation.surveyExtension.isValidEmail
import com.istnetworks.hivesdk.presentation.surveyExtension.isValidUrl
import java.util.*

/**validating the whole number + country code {+xxx,xxxxxxxxx}
 * {3} + {9,15}
 */
private const val CODE_AREA_PATTERN = "^[+][0-9]{2,3}\$"
private const val PHONE_NUMBER_PATTERN = "[0-9][0-9]{8,14}"
private const val POSTAL_CODE_PATTERN = "^\\d{5}(?:[-\\s]\\d{4})?\$"
private const val TAG = "Validator"
class Validator() {

    private var questionsList:MutableList<Question>?=null
    private var questionResponsesList:MutableList<QuestionResponses> = mutableListOf()

    fun setData(questionsList:MutableList<Question>?, questionResponsesList:MutableList<QuestionResponses>){
        this.questionsList=questionsList
        this.questionResponsesList=questionResponsesList
    }

    val showIsRequiredErrMsgLD = MutableLiveData<Pair<String, Boolean>>()
    val showNotValidErrMsgLD = MutableLiveData<Pair<String, Boolean>>()

    fun validateAnswer(currentQuestion:Question? ,responses: QuestionResponses?): Boolean {
        return if (responses==null && currentQuestion?.isRequired == false)
            true
        else
            isCurrentQuestionResponseValid(currentQuestion,responses)
    }

    private fun isCurrentQuestionResponseValid(currentQuestion: Question?,
                                               questionResponse: QuestionResponses?): Boolean {

        questionResponse?.let {
            return when (currentQuestion?.questionType) {
                QuestionType.EmailInput.value -> {
                    val validEmail = questionResponse.textResponse.isValidEmail()
                    showNotValidErrMsgLD.value =
                        Pair(currentQuestion.surveyQuestionGUID!!, !validEmail)
                    validEmail
                }
                QuestionType.URLInput.value -> {
                    val validUrl = questionResponse.textResponse?.isValidUrl()
                    showNotValidErrMsgLD.value =
                        Pair(currentQuestion.surveyQuestionGUID!!, validUrl == false)
                    validUrl == true
                }
                QuestionType.PhoneNumberInput.value -> {
                    val validPhone =
                        questionResponse.textResponse?.let {
                            PHONE_NUMBER_PATTERN.toRegex()
                                .matches(questionResponse.textResponse!!) && CODE_AREA_PATTERN.toRegex()
                                .matches(questionResponse.areaCode.toString())
                        }
                    showNotValidErrMsgLD.value =
                        Pair(currentQuestion.surveyQuestionGUID!!, validPhone == false)
                    validPhone == true

                }
                QuestionType.PostalCodeInput.value -> {
                    val validPostal =
                        questionResponse.textResponse?.matches(POSTAL_CODE_PATTERN.toRegex())
                    showNotValidErrMsgLD.value =
                        Pair(currentQuestion.surveyQuestionGUID!!, validPostal == false)
                    validPostal == true
                }
                else ->{
                    showIsRequiredErrMsgLD.value = Pair(currentQuestion?.surveyQuestionGUID!!,false)
                    true
                }
            }

        }

        showIsRequiredErrMsgLD.value = Pair(currentQuestion?.surveyQuestionGUID!!, true)
        return false

    }

    fun checkAllAnswersAreValid():Boolean{
        var isAllValid = true
        questionResponsesList.forEach {
            when (questionsList?.find { q-> it.questionGUID == q.surveyQuestionGUID }?.questionType) {
                QuestionType.EmailInput.value -> {
                    val validEmail = it.textResponse.isValidEmail()
                    showNotValidErrMsgLD.value = Pair(it.questionGUID!!,!validEmail)
                    if(isAllValid) {
                        isAllValid = validEmail
                    }
                }
                QuestionType.URLInput.value -> {
                    val validUrl = it.textResponse?.isValidUrl()
                    showNotValidErrMsgLD.value =
                        Pair(it.questionGUID!!,validUrl == false)
                    if(isAllValid) {
                        isAllValid = validUrl == true
                    }
                }
                QuestionType.PhoneNumberInput.value -> {
                    val validPhone =
                        it.let {
                            PHONE_NUMBER_PATTERN.toRegex()
                                .matches(it.textResponse!!) && CODE_AREA_PATTERN.toRegex()
                                .matches(it.areaCode.toString())
                        }
                    showNotValidErrMsgLD.value =
                        Pair(it.questionGUID!!, !validPhone)

                    if (isAllValid)
                    {
                        isAllValid =validPhone == true
                        it.textResponse.apply {
                            it.areaCode + this
                        }
                    }
                }
                QuestionType.PostalCodeInput.value->{
                    val validPostal =
                        it.textResponse?.matches(POSTAL_CODE_PATTERN.toRegex())
                    showNotValidErrMsgLD.value =
                        Pair(it.questionGUID!!,validPostal == false)
                    if(isAllValid) {
                        isAllValid = validPostal == true
                    }
                }
                else -> isAllValid =true
            }
        }
        return isAllValid
    }

    fun checkIsRequiredAnswered(previousQuestions: Stack<Int>,
                                lastQuestionAnsweredPosition:Int):Boolean{
        var isAllAnswered = true
        val isRequiredQuestionsList : List<Question> =
            questionsList!!.filter { it.isRequired==true }

        for(q in isRequiredQuestionsList)
        {
            val isAnswered = questionResponsesList.any { it.questionGUID == q.surveyQuestionGUID }
            val positionPushed = findQuestionPosition(q.surveyQuestionGUID)
            if (!isAnswered && (previousQuestions.contains(positionPushed) ||
                        positionPushed == lastQuestionAnsweredPosition)) {
                isAllAnswered = false
                showIsRequiredErrMsgLD.value = Pair(q.surveyQuestionGUID!!, true)
            }
        }

        return isAllAnswered
    }

    private fun findQuestionPosition(questionGUID: String?): Int =
        questionsList?.indexOfFirst { it.surveyQuestionGUID == questionGUID } ?: -1
}