package com.istnetworks.hivesdk.presentation.freeinputs

import android.os.Bundle
import android.text.InputFilter
import android.text.InputFilter.LengthFilter
import android.text.InputType
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.Keep
import androidx.annotation.NonNull
import androidx.annotation.StringRes
import androidx.core.content.ContextCompat
import androidx.core.os.bundleOf
import androidx.core.widget.doAfterTextChanged
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.toQuestionResponse
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentFreeInputsBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.interfaces.FullScreenQuestionNumberSetter
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.interfaces.SubmitButtonInterface
import com.istnetworks.hivesdk.presentation.interfaces.ValidationErrorInterface
import com.istnetworks.hivesdk.presentation.spinnerquestion.ARG_POSITION
import com.istnetworks.hivesdk.presentation.surveyExtension.questionTitleStyle


class FreeInputsFragment : BaseQuestionFragment(), IsRequiredInterface,
    ValidationErrorInterface, SubmitButtonInterface, FullScreenQuestionNumberSetter {

    private lateinit var binding: FragmentFreeInputsBinding
    private var questionNumber: Int = -1
    private var selectedQuestion: Question? = null
    private val position: Int? by lazy { arguments?.getInt(ARG_POSITION, -1) }
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentFreeInputsBinding.inflate(inflater)
        selectedQuestion = viewModel.findQuestion(position)
        stylingViews()
        initSubmitBtn()
        selectedQuestion?.questionType?.let { bindQuestions(it) }
        observeViewModel()
        return binding.root
    }


    private fun observeViewModel() {

    }

    private fun listenToInputTextChanges() {

        binding.hveEdtFreeInput.doAfterTextChanged {

            viewModel.updateQuestionResponsesList(
                selectedQuestion?.toQuestionResponse(
                    textResponse = it.toString(),
                    numberResponse = null
                )
            )
        }

        binding.hveEdtPhone.doAfterTextChanged {
            viewModel.updateQuestionResponsesList(
                selectedQuestion?.toQuestionResponse(
                    textResponse = it.toString(),
                    areaCode = binding.edtCountryCode.text.toString(),
                    numberResponse = null
                )
            )
        }
        binding.edtCountryCode.doAfterTextChanged {
            viewModel.updateQuestionResponsesList(
                selectedQuestion?.toQuestionResponse(
                    textResponse = binding.hveEdtPhone.text.toString(),
                    areaCode = it.toString(),
                    numberResponse = null
                )
            )
        }

    }

    override fun onResume() {
        super.onResume()
        bindQuestionTitle()
        listenToInputTextChanges()
        updatePagerHeight(binding.root)
        checkInputsAndIsRequiredError()
    }

    private fun checkInputsAndIsRequiredError() {
        val isRequired = selectedQuestion?.isRequired

        if (binding.hveEdtFreeInput.text.isEmpty() &&
            selectedQuestion?.questionType != QuestionType.PhoneNumberInput.value
            && isRequired == true
        ) {
            binding.tvErrorMessage.visibility = View.GONE
        }
        if (binding.hveEdtPhone.text.isEmpty() && binding.edtCountryCode.text.isEmpty()
            && isRequired == true
        ) {
            binding.tvErrorMessage.visibility = View.GONE
        }
    }

    private fun bindQuestions(questionType: Int) {
        binding.hveEdtFreeInput.visibility = View.VISIBLE
        binding.llPhone.visibility = View.GONE
        when (questionType) {
            QuestionType.TextInput.value -> {
                binding.hveEdtFreeInput.inputType = InputType.TYPE_CLASS_TEXT
                binding.hveEdtFreeInput.hint = getString(R.string.textInputPlaceholder)
                binding.hveEdtFreeInput.filters = arrayOf<InputFilter>(LengthFilter(100))
            }
            QuestionType.NumberInput.value -> {
                binding.hveEdtFreeInput.inputType = InputType.TYPE_CLASS_NUMBER
                binding.hveEdtFreeInput.hint = getString(R.string.numberInputPlaceholder)
                binding.hveEdtFreeInput.filters = arrayOf<InputFilter>(LengthFilter(30))
            }
            QuestionType.EmailInput.value -> {
                binding.hveEdtFreeInput.inputType = InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS
                binding.hveEdtFreeInput.hint = getString(R.string.emailInputPlaceholder)
                binding.hveEdtFreeInput.filters = arrayOf<InputFilter>(LengthFilter(100))
            }
            QuestionType.PhoneNumberInput.value -> {
                handlePhoneInput()
            }
            QuestionType.PostalCodeInput.value -> {
                binding.hveEdtFreeInput.inputType = InputType.TYPE_CLASS_NUMBER
                binding.hveEdtFreeInput.hint = getString(R.string.postalInputPlaceholder)
                binding.hveEdtFreeInput.filters = arrayOf<InputFilter>(LengthFilter(5))
            }
            QuestionType.URLInput.value -> {
                binding.hveEdtFreeInput.inputType = InputType.TYPE_TEXT_VARIATION_URI
                binding.hveEdtFreeInput.hint = getString(R.string.uRLInputPlaceholder)
                binding.hveEdtFreeInput.filters = arrayOf<InputFilter>(LengthFilter(100))
            }
        }
    }

    private fun handlePhoneInput() {
        binding.hveEdtFreeInput.visibility = View.GONE
        binding.llPhone.visibility = View.VISIBLE
        binding.hveEdtPhone.hint = getString(R.string.teleInputPlaceholder)
        binding.edtCountryCode.setTextColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.navyBlue
            )
        )
        binding.hveEdtPhone.setTextColor(ContextCompat.getColor(requireContext(), R.color.navyBlue))
    }

    private fun ShowProperErrorMessage(questionType: Int) {
        when (questionType) {
            QuestionType.EmailInput.value -> {
                setFreeInputValidationError(R.string.emailValidationMsg)
            }
            QuestionType.URLInput.value -> {
                setFreeInputValidationError(R.string.uRLValidationMsg)
            }
            QuestionType.PhoneNumberInput.value -> {
                setPhoneValidationError()
            }
            QuestionType.PostalCodeInput.value -> {
                setFreeInputValidationError(R.string.postalValidationMsg)
            }
            else -> {
                if (TextUtils.isEmpty(binding.hveEdtFreeInput.text)) {
                    setFreeInputValidationError(R.string.answer_not_valid)
                }
            }
        }
    }

    private fun setFreeInputValidationError(@StringRes messageId: Int) {
        binding.tvErrorMessage.text = getString(messageId)
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)
        binding.hveEdtFreeInput.setTextColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.errorColor
            )
        )
        binding.hveEdtFreeInput.background =
            activity?.let { it1 ->
                ContextCompat.getDrawable(
                    it1,
                    R.drawable.free_input_error
                )
            }

        updatePagerHeight(binding.root)
    }

    private fun hideFreeInputError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

        binding.hveEdtFreeInput.setTextColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.navyBlue
            )
        )
        binding.hveEdtFreeInput.background =
            activity?.let { it1 ->
                ContextCompat.getDrawable(
                    it1,
                    R.drawable.bg_free_input_rounded_border
                )
            }

        updatePagerHeight(binding.root)
    }

    private fun setPhoneValidationError() {
        binding.tvErrorMessage.text = getString(R.string.phoneValidationMsg)

        binding.tvErrorMessage.visibility = View.VISIBLE
        updatePagerHeight(binding.root)

        binding.hveEdtPhone.setTextColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.errorColor
            )
        )
        binding.llPhone.background =
            activity?.let { it1 ->
                ContextCompat.getDrawable(
                    it1,
                    R.drawable.free_input_error
                )
            }
        binding.edtCountryCode.setTextColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.errorColor
            )
        )
    }

    private fun hidePhoneError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

        binding.hveEdtPhone.setTextColor(ContextCompat.getColor(requireContext(), R.color.navyBlue))
        binding.llPhone.background =
            activity?.let { it1 ->
                ContextCompat.getDrawable(
                    it1,
                    R.drawable.bg_free_input_rounded_border
                )
            }

        binding.edtCountryCode.setTextColor(
            ContextCompat.getColor(
                requireContext(),
                R.color.navyBlue
            )
        )

        updatePagerHeight(binding.root)
    }

    private fun stylingViews() {
        val theme = viewModel.getSurveyTheme()
        binding.tvQuestionTitle.questionTitleStyle(theme?.questionTitleStyle)


    }

    private fun bindQuestionTitle() {
        binding.tvQuestionTitle.text =
            "${context?.getString(R.string.questionLabel)}${getQNumber()}. ${selectedQuestion?.title}"
    }

    private fun getQNumber(): Int {
        return if (viewModel.isFullScreen())
            questionNumber
        else
            viewModel.getPopupQuestionNumber()
    }

    private fun initSubmitBtn() {
        viewModel.setSubmitButtonBasedOnPosition(binding.hveBtnSubmit, position)
    }


    override fun showIsRequiredError() {
        binding.tvErrorMessage.text = getString(R.string.manadatoryQuestionMsg)
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)

    }

    override fun hideIsRequiredError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

    }

    @Keep
    companion object {
        @JvmStatic
        fun getInstance(@NonNull position: Int) =
            FreeInputsFragment().apply {
                arguments = bundleOf(ARG_POSITION to position)
            }
    }

    override fun showNotValidError(questionType: Int?) {
        if (questionType != null) {
            ShowProperErrorMessage(questionType)
            updatePagerHeight(binding.root)
        }

    }

    override fun hideNotValidError(questionType: Int?) {
        when (questionType) {
            QuestionType.PhoneNumberInput.value -> {
                hidePhoneError()
            }
            else -> {
                hideFreeInputError()
            }
        }
    }

    override fun showSubmitButton() {
        binding.hveBtnSubmit.show()
        updatePagerHeight(binding.root)
    }

    override fun hideSubmitButton() {
        binding.hveBtnSubmit.hide()
        updatePagerHeight(binding.root)
    }

    override fun setQuestionNumber(num: Int) {
        questionNumber = num
    }
}