package com.istnetworks.hivesdk.presentation.multiImageChoice

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.core.graphics.drawable.toDrawable
import androidx.lifecycle.lifecycleScope
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.Choices
import com.istnetworks.hivesdk.data.models.SelectedChoices
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.styles.TitleStyle
import com.istnetworks.hivesdk.data.models.response.toQuestionResponse
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentMultipleImageChoiceBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.interfaces.FullScreenQuestionNumberSetter
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.mainfragment.MainFragment
import com.istnetworks.hivesdk.presentation.surveyExtension.multiChoiceImageStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.multiChoiceStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.questionTitleStyle
import com.squareup.picasso.Picasso
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

private const val ARG_QUESTION_POSITION = "ARG_QUESTION_POSITION"
private const val TAG = "MultipleImageChoice"

class MultipleImageChoiceFragment : BaseQuestionFragment(), CompoundButton.OnCheckedChangeListener,
    IsRequiredInterface , FullScreenQuestionNumberSetter {
    private var questionPosition: Int? = null
    private var selectedQuestion: Question? = null
    private var isRequired: Boolean = false
    private var selectedChoices: ArrayList<SelectedChoices> = arrayListOf()
    private lateinit var binding: FragmentMultipleImageChoiceBinding
    private var questionNumber: Int = -1

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            questionPosition = it.getInt(ARG_QUESTION_POSITION, 0)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentMultipleImageChoiceBinding.inflate(inflater)

        observeViewModel()
        observeSurvey()
        onClickActions()
        initSubmitBtn()
        return binding.root
    }

    override fun onResume() {
        super.onResume()
        bindQuestionTitle()
        updatePagerHeight(binding.root)
        checkInputsAndIsRequiredError()
    }

    private fun checkInputsAndIsRequiredError(){
        if(selectedChoices.isEmpty())
        {
            binding.tvErrorMessage.visibility = View.GONE
        }

    }

    private fun initSubmitBtn() {
        viewModel.setSubmitButtonBasedOnPosition(binding.hveBtnSubmit,questionPosition)
    }

    private fun observeViewModel() {
        viewModel.showErrorMsg.observe(viewLifecycleOwner, {
            Toast.makeText(requireContext(), "$it", Toast.LENGTH_SHORT).show()
        })
    }

    private fun observeSurvey() {
        selectedQuestion = questionPosition?.let { viewModel.findQuestion(it) }
        binding.tvQuestionTitle.questionTitleStyle(viewModel.getSurveyTheme()?.questionTitleStyle)

        isRequired = selectedQuestion?.isRequired!!

        createChoices(selectedQuestion?.choices,viewModel.getSurveyTheme()?.questionChoicesStyle!!)

    }

    private fun bindQuestionTitle() {
        binding.tvQuestionTitle.text =
            "${context?.getString(R.string.questionLabel)}${getQNumber()}. ${selectedQuestion?.title}"
    }
    private fun getQNumber(): Int {
        return if (viewModel.isFullScreen())
            questionNumber
        else
            viewModel.getPopupQuestionNumber()
    }
    private fun createChoices(choiceList: List<Choices>?, style: TitleStyle) {
        val inflater = LayoutInflater.from(context)
        choiceList?.let {
            for (choice in it) {
                val cbChoice = inflater.inflate(
                    R.layout.multi_choice_image_item, binding.hveLiMultipleChoiceWrapper, false
                ) as CheckBox
                cbChoice.buttonDrawable = null
                cbChoice.id = choice.choiceID!!
                cbChoice.text = choice.title
                lifecycleScope.launch {
                    withContext(Dispatchers.IO)
                    {
                        try {

                            val bitmap =
                                Picasso.get().load(choice.imageURL)
                                    .placeholder(R.drawable.loading_animation)
                                    .resize(200, 200)
                                    .get().toDrawable(resources)
                            withContext(Dispatchers.Main) {

                                cbChoice.setCompoundDrawablesWithIntrinsicBounds(
                                    null,
                                    bitmap,
                                    null,
                                    null
                                )

                            }
                        } catch (e: Exception) {
                            Log.e(TAG, "createChoices: ", e)
                        }
                    }

                }

                cbChoice.multiChoiceImageStyle(
                    style, viewModel.getSurveyOptions()?.surveyTheme?.focusColor!!,
                    viewModel.getSurveyOptions()?.surveyTheme?.surveyBackgroundColor!!
                )
                cbChoice.setPadding(32, 32, 32, 16)
                (cbChoice.layoutParams as GridLayout.LayoutParams).setMargins(24, 32, 32, 8)
                cbChoice.setOnCheckedChangeListener(this)
                binding.hveLiMultipleChoiceWrapper.addView(cbChoice)
                this.view?.let {
                    updatePagerHeight(binding.root)
                }

            }
        }
    }
    private fun onClickActions() {


    }

    override fun onCheckedChanged(p0: CompoundButton?, p1: Boolean) {
        val checkedId = p0?.id
        val selectedChoice = selectedQuestion?.choices?.find { it.choiceID == checkedId }
        if (p1) {
            selectedChoices.add(SelectedChoices(selectedChoice?.choiceID, selectedChoice?.choiceGUID))
        }
        else
        {
            selectedChoices.removeIf { it -> it.choiceID == checkedId }
        }
        viewModel.updateQuestionResponsesList(
            selectedQuestion?.toQuestionResponse(
                "", null,
                selectedChoices
            )
        )
    }


    companion object {
        /**
         *
         * @param questionPosition Parameter 1.
         * @return A new instance of fragment SingleChoiceFragment.
         */

        @JvmStatic
        fun getInstance(questionPosition: Int) =
            MultipleImageChoiceFragment().apply {
                arguments = Bundle().apply {
                    putInt(ARG_QUESTION_POSITION, questionPosition)
                }
            }
    }
    override fun showIsRequiredError() {
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)

    }
  override fun setQuestionNumber(num: Int) {
        questionNumber=num
    }
    override fun hideIsRequiredError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

    }
}