package com.istnetworks.hivesdk.presentation.rating

import android.content.res.ColorStateList
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.NonNull
import androidx.core.os.bundleOf
import com.google.android.flexbox.FlexDirection
import com.google.android.flexbox.FlexWrap
import com.google.android.flexbox.FlexboxLayoutManager
import com.google.android.flexbox.JustifyContent
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.StarOption
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.toQuestionResponse
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.data.utils.StarOptionEnum
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentRatingBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.interfaces.FullScreenQuestionNumberSetter
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.interfaces.SubmitButtonInterface
import com.istnetworks.hivesdk.presentation.spinnerquestion.ARG_POSITION
import com.istnetworks.hivesdk.presentation.surveyExtension.questionTitleStyle

class RatingFragment : BaseQuestionFragment(), IsRequiredInterface, SubmitButtonInterface,
    FullScreenQuestionNumberSetter {

    private lateinit var binding: FragmentRatingBinding
    private var selectedQuestion: Question? = null
    private val position: Int? by lazy { arguments?.getInt(ARG_POSITION, -1) }
    private lateinit var selectedRating: HiveRatingBar
    private lateinit var underLineAdapter: UnderLineAdapter
    private var questionNumber: Int = -1

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentRatingBinding.inflate(inflater)
        selectedQuestion = viewModel.findQuestion(position)
        setupUnderLineRV()
        stylingViews()
        initSubmitBtn()
        selectedQuestion?.StarOption?.shape?.let { bindQuestions(it, selectedQuestion?.StarOption) }
        observeViewModel()
        return binding.root
    }

    private fun observeViewModel() {
    }

    override fun onResume() {
        super.onResume()
        bindQuestionTitle()
        updatePagerHeight(binding.root)
        position?.let { viewModel.updateSubmitBtnVisibilityBeforeAnswerChosen(it) }
        checkInputsAndIsRequiredError()
    }

    private fun checkInputsAndIsRequiredError(){
        if(selectedRating.rating == -1f&&
            selectedQuestion?.questionType != QuestionType.PhoneNumberInput.value)
        {
            binding.tvErrorMessage.visibility = View.GONE
        }
    }

    private fun bindQuestions(
        type: Int,
        starOption: StarOption?
    ) {

        when (type) {
            StarOptionEnum.HEART.value -> {
                selectedRating = binding.hveHeartRatingBar
            }
            StarOptionEnum.SMILEY.value -> {
                selectedRating = binding.hveSmileRatingBar
            }
            StarOptionEnum.THUMB.value -> {
                selectedRating = binding.hveThumbRatingBar
            }
            StarOptionEnum.STAR.value -> {
                selectedRating = binding.hveStarRatingBar
            }
        }

        selectedRating.show()
        selectedRating.setStarColor(
            ColorStateList.valueOf(
                Color.parseColor("#"+starOption?.fillColor)))
        listenToRating()
    }

    private fun stylingViews() {
        val theme = viewModel.getSurveyTheme()
        binding.tvQuestionTitle.questionTitleStyle(theme?.questionTitleStyle)


    }

    private fun bindQuestionTitle() {
        binding.tvQuestionTitle.text =
            "${context?.getString(R.string.questionLabel)}${getQNumber()}. ${selectedQuestion?.title}"
    }
    private fun getQNumber(): Int {
        return if (viewModel.isFullScreen())
            questionNumber
        else
            viewModel.getPopupQuestionNumber()
    }
    private fun listenToRating() {
        selectedRating.setOnRatingChangeListener { ratingBar, rating, fromUser ->
            setUnderLineAppearance(rating.toInt())
            viewModel.updateQuestionResponsesList(
                selectedQuestion?.toQuestionResponse(
                    "",
                    rating.toInt()
                )
            )
            position?.let { viewModel.updateSubmitBtnVisibilityBeforeAnswerChosen(it) }
        }
    }

    private fun setUnderLineAppearance(value: Int) {
        underLineAdapter.selectedPosition(value)
    }

    private fun setupUnderLineRV() {
        val flexAdapter = FlexboxLayoutManager(context, FlexDirection.ROW, FlexWrap.NOWRAP)
        flexAdapter.justifyContent = JustifyContent.SPACE_BETWEEN
        binding.hveRvUnderlineRating.setItemViewCacheSize(5);
        binding.hveRvUnderlineRating.setHasFixedSize(true);
        binding.hveRvUnderlineRating.layoutManager = flexAdapter
        underLineAdapter = UnderLineAdapter(requireContext(), listOf("1", "2", "3", "4", "5"),
            selectedQuestion?.StarOption?.fillColor ?: "#000")
        underLineAdapter.setHasStableIds(true)
        binding.hveRvUnderlineRating.adapter = underLineAdapter
    }

    private fun initSubmitBtn() {
        viewModel.setSubmitButtonBasedOnPosition(binding.hveBtnSubmit, position)
    }

    companion object {
        @JvmStatic
        fun getInstance(@NonNull position: Int) =
            RatingFragment().apply {
                arguments = bundleOf(ARG_POSITION to position)
            }
    }
    override fun showIsRequiredError() {
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)

    }

    override fun hideIsRequiredError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

    }

    override fun showSubmitButton() {
        binding.hveBtnSubmit.show()
        updatePagerHeight(binding.root)
    }

    override fun setQuestionNumber(num: Int) {
        questionNumber = num
    }

    override fun hideSubmitButton() {
        binding.hveBtnSubmit.hide()
        updatePagerHeight(binding.root)
    }
}