package com.istnetworks.hivesdk.presentation.singleChoice

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RadioButton
import android.widget.Toast
import androidx.core.content.ContextCompat
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.Choices
import com.istnetworks.hivesdk.data.models.SelectedChoices
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.styles.TitleStyle
import com.istnetworks.hivesdk.data.models.response.toQuestionResponse
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentSingleChoiceBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.interfaces.FullScreenQuestionNumberSetter
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.interfaces.SubmitButtonInterface
import com.istnetworks.hivesdk.presentation.surveyExtension.questionTitleStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.radioButtonColorStateDrawable
import com.istnetworks.hivesdk.presentation.surveyExtension.singleChoiceStyle

private const val ARG_QUESTION_POSITION = "ARG_QUESTION_POSITION"
private const val TAG = "SingleChoiceFragment"

class SingleChoiceFragment : BaseQuestionFragment(), IsRequiredInterface, SubmitButtonInterface,
    FullScreenQuestionNumberSetter {
    private var questionPosition: Int? = null
    private var selectedQuestion: Question? = null
    private var isRequired: Boolean = false
    private var selectedChoice: Int = 0
    private lateinit var binding: FragmentSingleChoiceBinding
    private var questionNumber: Int = -1

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            questionPosition = it.getInt(ARG_QUESTION_POSITION, 0)
        }
    }

    override fun onResume() {
        super.onResume()
        bindQuestionTitle()
        updatePagerHeight(binding.root)
        questionPosition?.let { viewModel.updateSubmitBtnVisibilityBeforeAnswerChosen(it) }
        checkInputsAndIsRequiredError()
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentSingleChoiceBinding.inflate(inflater)

        observeViewModel()
        initSubmitBtn()
        onClickActions()
        observeSurvey()
        return binding.root
    }

    private fun checkInputsAndIsRequiredError(){
        if(binding.rgSingleChoiceWrapper.checkedRadioButtonId == -1)
        {
            binding.tvErrorMessage.visibility = View.GONE
        }

    }
    private fun initSubmitBtn() {
        viewModel.setSubmitButtonBasedOnPosition(binding.hveBtnSubmit,questionPosition)
    }

    private fun observeViewModel() {
        viewModel.showErrorMsg.observe(viewLifecycleOwner, {
            Toast.makeText(requireContext(), "$it", Toast.LENGTH_SHORT).show()
        })

    }

    private fun observeSurvey() {
        selectedQuestion = questionPosition?.let { viewModel.findQuestion(it) }
        binding.tvQuestionTitle.questionTitleStyle(viewModel.survey?.surveyOptions?.surveyTheme?.questionTitleStyle)
        isRequired = selectedQuestion?.isRequired!!

        createChoices(selectedQuestion?.choices,
            viewModel.survey?.surveyOptions?.surveyTheme?.questionChoicesStyle!!)

    }

    private fun bindQuestionTitle() {
        binding.tvQuestionTitle.text =
            "${context?.getString(R.string.questionLabel)}${getQNumber()}. ${selectedQuestion?.title}"
    }
    private fun getQNumber(): Int {
        return if (viewModel.isFullScreen())
            questionNumber
        else
            viewModel.getPopupQuestionNumber()
    }
    private fun createChoices(choiceList: List<Choices>?,style:TitleStyle){
        val inflater = LayoutInflater.from(context)
        choiceList?.let {
            for (choice in it) {
                val rbChoice = inflater.inflate(
                    R.layout.single_choice_item, binding.rgSingleChoiceWrapper, false
                ) as RadioButton

                rbChoice.id = choice.choiceID!!
                rbChoice.text = choice.title

                rbChoice.buttonDrawable = radioButtonColorStateDrawable(
                    ContextCompat.getDrawable(
                        requireContext(),
                        R.drawable.radiobutton_notchecked
                    )!!,
                    ContextCompat.getDrawable(requireContext(), R.drawable.radiobutton_checked)!!,
                )

                rbChoice.singleChoiceStyle(
                    style, viewModel.getSurveyOptions()?.surveyTheme?.focusColor!!,
                    viewModel.getSurveyOptions()?.surveyTheme?.surveyBackgroundColor!!
                )
                binding.rgSingleChoiceWrapper.addView(rbChoice)

            }
        }
    }

    private fun onClickActions() {


        binding.rgSingleChoiceWrapper.setOnCheckedChangeListener { radioGroup, i ->
            val checkedId = radioGroup.checkedRadioButtonId
            val selectedChoice = selectedQuestion?.choices?.find { it.choiceID == checkedId }
            viewModel.updateQuestionResponsesList(
                selectedQuestion?.toQuestionResponse("",null,
                listOf(SelectedChoices(selectedChoice?.choiceID,selectedChoice?.choiceGUID)
                ),choiceGUID = selectedChoice?.choiceGUID
                )
            )
            questionPosition?.let { viewModel.updateSubmitBtnVisibilityBeforeAnswerChosen(it) }
        }


    }

    companion object {
        /**
         *
         * @param questionPosition Parameter 1.
         * @return A new instance of fragment SingleChoiceFragment.
         */
        @JvmStatic
        fun newInstance(questionPosition: Int) =
            SingleChoiceFragment().apply {
                arguments = Bundle().apply {
                    putInt(ARG_QUESTION_POSITION, questionPosition)
                }
            }
    }
    override fun showIsRequiredError() {
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)

    }

    override fun hideIsRequiredError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

    }

    override fun showSubmitButton() {
        binding.hveBtnSubmit.show()
        updatePagerHeight(binding.root)
    }

    override fun setQuestionNumber(num: Int) {
        questionNumber = num
    }

    override fun hideSubmitButton() {
        binding.hveBtnSubmit.hide()
        updatePagerHeight(binding.root)
    }
}