package com.istnetworks.hivesdk.presentation.singleImageChoice

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.RadioButton
import android.widget.Toast
import androidx.core.graphics.drawable.toDrawable
import androidx.lifecycle.lifecycleScope
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.Choices
import com.istnetworks.hivesdk.data.models.SelectedChoices
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.styles.TitleStyle
import com.istnetworks.hivesdk.data.models.response.toQuestionResponse
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentSingleImageChoiceBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.interfaces.FullScreenQuestionNumberSetter
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.interfaces.SubmitButtonInterface
import com.istnetworks.hivesdk.presentation.surveyExtension.questionTitleStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.singleChoiceImageStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.singleChoiceStyle
import com.squareup.picasso.Picasso
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext


private const val ARG_QUESTION_POSITION = "ARG_QUESTION_POSITION"
private const val TAG = "SingleImageChoice"

class SingleImageChoiceFragment : BaseQuestionFragment() ,IsRequiredInterface,SubmitButtonInterface,
    FullScreenQuestionNumberSetter {
    private var questionPosition: Int? = null
    private var selectedQuestion: Question? = null
    private var isRequired: Boolean = false
    private lateinit var binding: FragmentSingleImageChoiceBinding
    private var questionNumber: Int = -1


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            questionPosition = it.getInt(ARG_QUESTION_POSITION, 0)
        }
    }

    override fun onResume() {
        super.onResume()
        bindQuestionTitle()
        updatePagerHeight(binding.root)
        checkInputsAndIsRequiredError()
        questionPosition?.let { viewModel.updateSubmitBtnVisibilityBeforeAnswerChosen(it) }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        binding = FragmentSingleImageChoiceBinding.inflate(inflater)

        observeViewModel()
        onClickActions()
        observeSurvey()
        initSubmitBtn()
        return binding.root
    }

    private fun checkInputsAndIsRequiredError(){
        if(binding.hveRgSingleChoiceWrapper.checkedCheckableImageButtonId == -1)
        {
            binding.tvErrorMessage.visibility = View.GONE
        }

    }
    private fun initSubmitBtn() {
        viewModel.setSubmitButtonBasedOnPosition(binding.hveBtnSubmit,questionPosition)
    }

    private fun observeViewModel() {
        viewModel.showErrorMsg.observe(viewLifecycleOwner, {
            Toast.makeText(requireContext(), "$it", Toast.LENGTH_SHORT).show()
        })
    }


    private fun observeSurvey() {

        selectedQuestion = questionPosition?.let { viewModel.findQuestion(it) }
        binding.hveTvQuestionTitle.questionTitleStyle(viewModel.survey?.surveyOptions?.surveyTheme?.questionTitleStyle)

        isRequired = selectedQuestion?.isRequired!!

        createChoices(selectedQuestion?.choices,viewModel?.survey?.surveyOptions?.
        surveyTheme?.questionChoicesStyle!!)

    }

    private fun bindQuestionTitle() {
        binding.hveTvQuestionTitle.text =
            "${context?.getString(R.string.questionLabel)}${getQNumber()}. ${selectedQuestion?.title}"
    }
    private fun getQNumber(): Int {
        return if (viewModel.isFullScreen())
            questionNumber
        else
            viewModel.getPopupQuestionNumber()
    }
    private fun createChoices(choiceList: List<Choices>?, style: TitleStyle){
        val inflater = LayoutInflater.from(context)
        choiceList?.let {
            for (choice in it) {
                val rbChoice = inflater.inflate(
                    R.layout.single_choice_image_item, binding.hveRgSingleChoiceWrapper, false
                ) as RadioButton

                rbChoice.id = choice.choiceID!!
                rbChoice.text = choice.title
                lifecycleScope.launch {
                    withContext(Dispatchers.IO)
                    {
                        try {

                            val bitmap =
                                Picasso.get().load(choice.imageURL)
                                    .placeholder(R.drawable.loading_animation)
                                    .resize(200, 200)
                                    .get().toDrawable(resources)
                            withContext(Dispatchers.Main) {

                                rbChoice.setCompoundDrawablesWithIntrinsicBounds(
                                    null,
                                    bitmap,
                                    null,
                                    null
                                )


                                updatePagerHeight(binding.root)
                            }
                        } catch (e: Exception) {
                            Log.e(TAG, "createChoices: ", e)
                        }
                    }

                }
                rbChoice.singleChoiceImageStyle(
                    style, viewModel.getSurveyOptions()?.surveyTheme?.focusColor!!,
                    viewModel.getSurveyOptions()?.surveyTheme?.surveyBackgroundColor!!
                )
                rbChoice.setPadding(32, 32, 32, 16)
                (rbChoice.layoutParams as LinearLayout.LayoutParams).setMargins(24, 32, 44, 8)
                binding.hveRgSingleChoiceWrapper.addView(rbChoice)
                this.view?.let {
                    updatePagerHeight(binding.root)
                }

            }
        }
    }


    private fun onClickActions() {


        binding.hveRgSingleChoiceWrapper.setOnCheckedChangeListener { radioGroup, i ->
            val checkedId = radioGroup.checkedCheckableImageButtonId
            val selectedChoice = selectedQuestion?.choices?.find { it.choiceID == checkedId }
            viewModel.updateQuestionResponsesList(
                selectedQuestion?.toQuestionResponse(
                    "", 0,
                    listOf(
                        SelectedChoices(selectedChoice?.choiceID, selectedChoice?.choiceGUID)
                    ),choiceGUID = selectedChoice?.choiceGUID
                )
            )

            questionPosition?.let { viewModel.updateSubmitBtnVisibilityBeforeAnswerChosen(it) }
        }


    }

    companion object {
        /**
         *
         * @param questionPosition Parameter 1.
         * @return A new instance of fragment SingleChoiceFragment.
         */
        @JvmStatic
        fun getInstance(questionPosition: Int) =
            SingleImageChoiceFragment().apply {
                arguments = Bundle().apply {
                    putInt(ARG_QUESTION_POSITION, questionPosition)
                }
            }
    }
    override fun showIsRequiredError() {
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)

    }

    override fun hideIsRequiredError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

    }

    override fun showSubmitButton() {
        binding.hveBtnSubmit.show()
        updatePagerHeight(binding.root)
    }
  override fun setQuestionNumber(num: Int) {
        questionNumber=num
    }
    override fun hideSubmitButton() {
        binding.hveBtnSubmit.hide()
        updatePagerHeight(binding.root)
    }
}