package com.istnetworks.hivesdk.presentation.viewmodel

import android.graphics.Color
import android.util.Log
import androidx.core.util.Pair
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.android.material.button.MaterialButton
import com.istnetworks.hivesdk.data.models.QuestionResponses
import com.istnetworks.hivesdk.data.models.RelevantWebSurveyBody
import com.istnetworks.hivesdk.data.models.RelevantWebSurveyResponse
import com.istnetworks.hivesdk.data.models.Status
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.Survey
import com.istnetworks.hivesdk.data.models.response.toSaveSurveyBody
import com.istnetworks.hivesdk.data.repository.HiveSDKRepository
import com.istnetworks.hivesdk.data.utils.HiveSDKType
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.data.utils.extensions.*
import com.istnetworks.hivesdk.domain.usecases.SkipLogicHandler
import com.istnetworks.hivesdk.domain.usecases.Validator
import com.istnetworks.hivesdk.presentation.surveyExtension.submitButtonStyle
import kotlinx.coroutines.launch
import java.util.*
import kotlin.math.roundToInt


class HiveSDKViewModel(
    private val hiveSDKRepository: HiveSDKRepository,
    private val skipHandler: SkipLogicHandler,
    private val validator: Validator
) : ViewModel() {

    val getSurveyResponseLD = MutableLiveData<RelevantWebSurveyResponse?>()
    var survey: Survey? = null
    val saveSurveyResponseLD = MutableLiveData<RelevantWebSurveyResponse?>()
    val isLoading = MutableLiveData<Boolean>()
    val showErrorMsg = MutableLiveData<String?>()
     val questionResponsesList: MutableList<QuestionResponses> = mutableListOf()
    val updateProgressSliderLD = MutableLiveData<Float>()
    private var previousQuestions = Stack<Int>()
    var showSubmitButtonLD = MutableLiveData<Pair<Int, Boolean>>()
    var enableNextButtonLD = MutableLiveData<Boolean>()
    var resetNextViewPagerFragmentsLD = MutableLiveData<List<Int>>()
    val nextPositionLD = MutableLiveData<Int>()
    var lastAnsweredQuestionPosition: Int = -1

    fun findQuestion(position: Int?): Question? {
        if (position == null || position == -1) return null
        return survey?.questions?.get(position)
    }

    fun findQuestionPosition(questionGUID: String?): Int =
        survey?.questions?.indexOfFirst { it.surveyQuestionGUID == questionGUID } ?: -1

    fun getPopupQuestionNumber(): Int {
        return previousQuestions.size + 1
    }

    /**
     * @return if no skip logic return next position
     * else if has skip logic return SkipTo
     * if skip to end return -1
     */
    fun getSkipToQuestionPosition(currentQuestionPosition: Int): Int {
        val response: QuestionResponses? = getQuestionResponseByPosition(currentQuestionPosition)
        return skipHandler.getSkipToQuestionPosition(currentQuestionPosition,response)
    }

    fun surveyBackgroundColor() = Color.parseColor("#" + getSurveyTheme()?.surveyBackgroundColor)

    fun isFullScreen(): Boolean {
        return getSurveyResponseLD.value?.survey?.surveyOptions?.displayMode == HiveSDKType.FULLSCREEN.value
    }
    fun isPopup(): Boolean {
        return getSurveyResponseLD.value?.survey?.surveyOptions?.displayMode == HiveSDKType.POPUP.value
    }
    fun isBottomSheet(): Boolean {
        return getSurveyResponseLD.value?.survey?.surveyOptions?.displayMode == HiveSDKType.BOTTOM_CARD.value
    }

    fun updateSubmitBtnVisibilityBeforeAnswerChosen(questionPosition: Int) {
        if (isCouldBeLastQuestion(questionPosition))
            showSubmitButtonLD.value = Pair(questionPosition,true)
    }

    fun isCouldBeLastQuestion(questionPosition: Int): Boolean {

        val question = findQuestion(questionPosition)
        val hasAnswer = hasQuestionResponse(question!!.surveyQuestionID!!)
        val hasSkipLogic = skipHandler.hasSkipLogic(question.surveyQuestionGUID!!)
        return hasSkipLogic && !hasAnswer && isSingleChoiceModeQuestion(question)

    }

    /**
     * @returns list of the questions that should appear to the user
     */
    fun getEligibleQuestions(): MutableList<Question> {
        val questions = mutableListOf<Question>()
        var position = 0
        val allQuestions = survey?.questions
        while (position in 0 until allQuestions?.size!!) {
            questions.add(allQuestions[position])
            if (isCouldBeLastQuestion(position))
                break
            val skipTo = getSkipToQuestionPosition(position)
            if (skipTo != -1)
                position = skipTo
            else
                break
        }
        return questions
    }

    /**
     * @param qGuid : Question Guid
     *  Update in the following fragment
     *  for Next button and Submit button based on Skip logic
     *  (POPUP / Bottom)
     */
    fun updateNextArrowEnablingOnFragmentChange(questionPosition: Int) {
        if (questionPosition == survey?.questions?.lastIndex ) {
            enableNextButtonLD.value = false
            return
        }
        val question = survey?.questions?.get(questionPosition)
        val hasAnswer = hasQuestionResponse(question!!.surveyQuestionID!!)
        val hasSkipLogic = skipHandler.hasSkipLogic(question.surveyQuestionGUID!!)
        val skipTo = getSkipToQuestionPosition(questionPosition)
        if (isSingleChoiceModeQuestion(question))
            if (hasSkipLogic)
                if (hasAnswer)
                    enableNextButtonLD.value = skipTo != -1
                else
                    enableNextButtonLD.value = false
            else
                enableNextButtonLD.value = true
        else
            enableNextButtonLD.value = true


    }

    /**
     * @param qGuid : Question Guid
     *  Update in the same fragment
     *  for Next button and Submit button based on Skip logic
     *  (All Survey Types)
     */
    fun updateSubmitAndNxtAfterAnswerChosen(qGuid: String?) {
        if (qGuid == null) return
        val questionPosition = findQuestionPosition(qGuid)
        if (questionPosition == -1) return
        if (questionPosition == survey?.questions?.lastIndex) {
            enableNextButtonLD.value = false
            return
        }
        if (skipHandler.hasSkipLogic(qGuid)) {
            val skipTo = getSkipToQuestionPosition(questionPosition)
            if (skipTo == -1) //skip to end
            {
                showSubmitButtonLD.value = Pair(questionPosition, true)
                enableNextButtonLD.value = false
            } else if (skipTo != -1) {
                showSubmitButtonLD.value = Pair(questionPosition,false)
                enableNextButtonLD.value = true
            }

        } else {
            enableNextButtonLD.value = true
        }

    }

    fun isSingleChoiceModeQuestion(question: Question?) =  (question?.questionType == QuestionType.SingleChoice.value
                || question?.questionType == QuestionType.StarQuestion.value
                || question?.questionType == QuestionType.SlideQuestion.value
                || question?.questionType == QuestionType.NPS.value
                || question?.questionType == QuestionType.CSAT.value
                || question?.questionType == QuestionType.ListQuestion.value
                || question?.questionType == QuestionType.Emoji.value
                || question?.questionType == QuestionType.ImageSingleChoice.value)

    fun getQuestionResponseByPosition(current: Int): QuestionResponses? {
        val currentQuestion = survey?.questions?.get(current)

        return questionResponsesList.find { it.questionID == currentQuestion?.surveyQuestionID }
    }

    fun hasQuestionResponse(questionId: Int?): Boolean {
        return questionResponsesList.any { it.questionID == questionId }
    }

    fun getSurvey(username: String, password: String,lang:String?) = viewModelScope.launch {
        val surveyBody = RelevantWebSurveyBody()
        surveyBody.language = lang
        surveyBody.channel = "InApp"
        surveyBody.customerName = "saad"
        surveyBody.customerEmail = "ss@ss.com"
        surveyBody.customerPhone = "01234567890"
        surveyBody.dispositionCodes = listOf("complian Event")

        val surveyResult =
            hiveSDKRepository.getRelevantWebSurveyResource(username, password, surveyBody)
        when (surveyResult.status) {
            Status.SUCCESS -> {
                getSurveyResponseLD.value = surveyResult.data
                isLoading.value = false
            }
            Status.ERROR -> {
                showErrorMsg.value = surveyResult.message
                isLoading.value = false
            }
            Status.LOADING -> isLoading.value = true
        }

        survey = surveyResult.data?.survey
        skipHandler.setData(survey?.skipLogic, survey?.questions)
        validator.setData(survey?.questions, questionResponsesList)
    }

    fun updateQuestionResponsesList(questionResponse: QuestionResponses?) {
        questionResponse?.let { q ->
            val duplicatedResponseObj = questionResponsesList.find { it.questionID == q.questionID }
            val question = findQuestion(findQuestionPosition(q.questionGUID))
            shouldResetNextFragments(duplicatedResponseObj, questionResponse)
            if (duplicatedResponseObj != null)
                questionResponsesList.remove(duplicatedResponseObj)

            if (isEmptyResponse(questionResponse))
                return@let
            questionResponsesList.add(q)
            validator.validateAnswer(question,q)
            updateFullScreenSkipLogic(q)
            updateSubmitAndNxtAfterAnswerChosen(q.questionGUID)
        }
        updateProgressBar()
    }

    fun getIsRequiredErrorLD() = validator.showIsRequiredErrMsgLD

    fun getIsValidationErrorLD() = validator.showNotValidErrMsgLD

    private fun shouldResetNextFragments(
        duplicatedResponseObj: QuestionResponses?,
        questionResponse: QuestionResponses
    ) {
        if (isBottomSheet() || isPopup()) {
            if (duplicatedResponseObj?.equals(questionResponse) == false) {
                if(isSingleChoiceModeQuestion(findQuestion(findQuestionPosition(questionResponse.questionGUID))))
                resetNextViewPagerFragmentsLD.value = getListOfResetedPositions(questionResponse)
                    Log.d("TAG", "shouldResetNextFragments: ${getListOfResetedPositions(questionResponse)}")
                }
            }
        }

    private fun updateFullScreenSkipLogic(q: QuestionResponses) {
        lastAnsweredQuestionPosition = findQuestionPosition(q.questionGUID)
        nextPositionLD.value = getSkipToQuestionPosition(findQuestionPosition(q.questionGUID))
    }

    private fun getListOfResetedPositions(questionResponse: QuestionResponses): List<Int> {
        val questionPosition = findQuestionPosition(questionResponse.questionGUID)
        val listOfResetedPositions = mutableListOf<Int>()
        questionResponsesList.forEach {
            val p = findQuestionPosition(it.questionGUID)
            if (p > questionPosition)
                listOfResetedPositions.add(p)
        }

        return listOfResetedPositions
    }

    private fun updateProgressBar() {
        val eligibleQuestionsLength = getEligibleQuestions().size


        val valueToFloat =
            (questionResponsesList.size.toFloat() / eligibleQuestionsLength) * survey?.questions?.size!!
        val valueToInt = valueToFloat.roundToInt()

        updateProgressSliderLD.value = valueToInt.toFloat()


    }

    private fun isEmptyResponse(question: QuestionResponses?) = (question?.numberResponse == null
                && question?.textResponse.isNullOrEmpty()
                && question?.selectedChoices.isNullOrEmpty()
                && question?.areaCode.isNullOrEmpty())

    private fun saveSurvey() {
        viewModelScope.launch {
            val resp = hiveSDKRepository.saveWebSurvey(generateSaveSurveyRequest())
            when (resp.status) {
                Status.SUCCESS -> saveSurveyResponseLD.value = resp.data
                Status.ERROR -> showErrorMsg.value = resp.message
                Status.LOADING -> TODO()
            }
        }
    }

    fun getSurveyTheme() = getSurveyOptions()?.surveyTheme

    fun getSurveyOptions() = survey?.surveyOptions

    private fun generateSaveSurveyRequest() =
        survey?.toSaveSurveyBody(questionResponsesList)

    fun hasProgressBar(): Boolean? {
        return getSurveyResponseLD.value?.survey?.surveyOptions?.hasProgressBar
    }

    fun validateAnswer(questionPosition: Int): Boolean {
        val currentQuestion = findQuestion(questionPosition)
        val questionResponse = getQuestionResponseByPosition(questionPosition)
        return validator.validateAnswer(currentQuestion,questionResponse)
    }

    fun getAndPopPreviousPosition(): Int {

        return if (previousQuestions.size > 0) {
            previousQuestions.pop()
        } else {
            return 0
        }
    }

    fun pushToPreviousQuestionsStack(currentQuestionNumber: Int) {
        previousQuestions.push(currentQuestionNumber)
    }

    fun setSubmitButtonBasedOnPosition(btn: MaterialButton, questionPosition: Int?) {
        btn.submitButtonStyle(getSurveyTheme()?.submitButton)
        if (survey?.questions?.lastIndex == questionPosition) {
            btn.show()
        } else {
            btn.hide()
        }
        val question = findQuestion(questionPosition)
        if (question?.isRequired == true && hasQuestionResponse(question.surveyQuestionID)) {
            btn.disable()
        } else {
            btn.enable()
        }
        btn.tag = questionPosition
        btn.onClick {
            if(validator.checkIsRequiredAnswered(previousQuestions,(btn.tag as Int))
                && validator.checkAllAnswersAreValid()) {
                saveSurvey()
            }
        }
    }

    fun removeResponseByQuestionGuid(qGUID: String?) {
        qGUID?.let { qGuid ->
            val resp = questionResponsesList.find { it.questionGUID == qGuid }
            questionResponsesList.remove(resp)
        }
    }

}