package com.istnetworks.hivesdk.presentation.viewmodel

import androidx.arch.core.executor.testing.InstantTaskExecutorRule
import androidx.core.util.Pair
import com.istnetworks.hivesdk.data.models.*
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.Survey
import com.istnetworks.hivesdk.data.repository.HiveSDKRepository
import com.istnetworks.hivesdk.data.repository.HiveSDKRepositoryImpl
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.domain.usecases.SkipLogicHandler
import com.istnetworks.hivesdk.domain.usecases.Validator
import com.istnetworks.hivesdk.utils.CoroutineTestRule
import com.istnetworks.hivesdk.utils.observeOnce
import io.mockk.coEvery
import io.mockk.every
import io.mockk.mockk
import io.mockk.spyk
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.runBlockingTest
import org.junit.Assert.assertEquals
import org.junit.Before
import org.junit.Rule
import org.junit.Test

@ExperimentalCoroutinesApi
class HiveSDKViewModelTest {
    @get:Rule
    val rule = InstantTaskExecutorRule()

    @get:Rule
    var coroutinesTestRule = CoroutineTestRule()

    private lateinit var viewModel: HiveSDKViewModel
    private lateinit var repo: HiveSDKRepository
    private lateinit var skipLogicHandler: SkipLogicHandler
    private lateinit var question: Question
    private lateinit var validator: Validator
    private lateinit var responseBody: Resource<RelevantWebSurveyResponse>

    @Before
    fun setUp() {
        repo = mockk()
        skipLogicHandler = spyk()
        validator = Validator()
        responseBody = mockk()
        viewModel = HiveSDKViewModel(repo, skipLogicHandler, validator)
        viewModel.survey = Survey(questions = getQuestionList(), skipLogic = getSkipLogic())

    }

    @Test
    fun `getSurvey  successful Response`() {
        runBlockingTest {
            val surveyBody = getRequestBody()
            responseBody = Resource.success(RelevantWebSurveyResponse(isSuccess = true))
            val r = getMockedRepo(responseBody, surveyBody)
            viewModel = HiveSDKViewModel(r, skipLogicHandler, validator)
            viewModel.getSurvey("saad", "InApp2021", "lang")
            var responseEntity: RelevantWebSurveyResponse? = null

            viewModel.getSurveyResponseLD.observeOnce {
                responseEntity = it
            }
            assertEquals(responseEntity?.isSuccess, true)
        }
    }
    @Test
    fun `getSurvey  error showmessage`() {
        runBlockingTest {
            val surveyBody = getRequestBody()
            responseBody = Resource.error(msg = "m", null)
            val r = getMockedRepo(responseBody, surveyBody)
            viewModel = HiveSDKViewModel(r, skipLogicHandler, validator)
            viewModel.getSurvey("saad", "InApp2021", "lang")
            var msg = ""
            viewModel.showErrorMsg.observeOnce {
                msg = it ?: ""
            }
            assertEquals(msg, "m")
        }
    }

    @Test
    fun `isCouldBeLastQuestion  returns true `() {
        //setup data
        question = Question("guid", 12, "t", true, QuestionType.CSAT.value)
        viewModel.survey = Survey(questions = mutableListOf(question))
        every { skipLogicHandler.hasSkipLogic(question.surveyQuestionGUID) } returns true
        // call
        val bool = viewModel.isCouldBeLastQuestion(0)
        //verify
        assertEquals(bool, true)
    }

    @Test
    fun `isCouldBeLastQuestion  returns false as it doesnot have SkipLogic `() {
        //setup data
        question = Question("guid", 12, "t", true, QuestionType.CSAT.value)
        viewModel.survey = Survey(questions = mutableListOf(question))
        every { skipLogicHandler.hasSkipLogic(question.surveyQuestionGUID) } returns false
        // call
        val bool = viewModel.isCouldBeLastQuestion(0)
        //verify
        assertEquals(bool, false)
    }

    @Test
    fun `isCouldBeLastQuestion  returns false as it has answers `() {
        //setup data
        question = Question("guid", 12, "t", true, QuestionType.CSAT.value)
        viewModel.survey = Survey(questions = mutableListOf(question))
        viewModel.updateQuestionResponsesList(
            QuestionResponses(
                question.surveyQuestionID,
                question.surveyQuestionGUID,
                "tt"
            )
        )
        every { skipLogicHandler.hasSkipLogic(question.surveyQuestionGUID) } returns true
        // call
        val bool = viewModel.isCouldBeLastQuestion(0)
        //verify
        assertEquals(bool, false)
    }

    @Test
    fun `getEligibleQuestions  returns all questions as no skip logic`() {
        //setup data
        viewModel.survey!!.questions!!.forEachIndexed { i, it ->
            every { skipLogicHandler.hasSkipLogic(it.surveyQuestionGUID) } returns false
            every { skipLogicHandler.getSkipToQuestionPosition(i, null) } returns i + 1
        }
        // call
        val list = viewModel.getEligibleQuestions()
        //verify
        assertEquals(getQuestionList().size, list.size)
    }

    @Test
    fun `getEligibleQuestions  returns 1 as first question could be the last`() {
        //setup data
        val firstQuestion = viewModel.survey!!.questions!![0]
        val copiedQuestion = firstQuestion.copy(questionType = QuestionType.CSAT.value)
        viewModel.survey!!.questions!![0] = copiedQuestion
        every { skipLogicHandler.hasSkipLogic(copiedQuestion.surveyQuestionGUID) } returns true
        // call
        val list = viewModel.getEligibleQuestions()
        //verify
        assertEquals(1, list.size)
    }

    @Test
    fun `getEligibleQuestions  returns 1 as first question skip to end`() {
        //setup data
        val firstQuestion = viewModel.survey!!.questions!![0]
        every { skipLogicHandler.hasSkipLogic(firstQuestion.surveyQuestionGUID) } returns false
        every { skipLogicHandler.getSkipToQuestionPosition(0, null) } returns -1
        // call
        val list = viewModel.getEligibleQuestions()
        //verify
        assertEquals(1, list.size)
    }


    @Test
    fun `updateNextArrowEnablingOnFragmentChange  disable as it is last position`() {
        //setup data
        // call
        viewModel.updateNextArrowEnablingOnFragmentChange(viewModel.survey!!.questions!!.lastIndex)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(false, t)
    }

    @Test
    fun `updateNextArrowEnablingOnFragmentChange  enabled as it is not SCM`() {
        //setup data
        /* val firstQuestion = viewModel.survey!!.questions!![0]
         every { skipLogicHandler.hasSkipLogic(firstQuestion.surveyQuestionGUID) } returns false
         every { skipLogicHandler.getSkipToQuestionPosition(0,null) } returns -1*/
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val copiedVersion = questionUnderTest.copy(questionType = QuestionType.EmailInput.value)
        viewModel.survey!!.questions!!.set(2, copiedVersion)
        // call
        viewModel.updateNextArrowEnablingOnFragmentChange(2)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(true, t)
    }

    @Test
    fun `updateNextArrowEnablingOnFragmentChange  enabled as it is SCM but no skip logic`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val copiedVersion = questionUnderTest.copy(questionType = QuestionType.SingleChoice.value)
        every { skipLogicHandler.hasSkipLogic(copiedVersion.surveyQuestionGUID) } returns false
        viewModel.survey!!.questions!![2] = copiedVersion
        // call
        viewModel.updateNextArrowEnablingOnFragmentChange(2)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(true, t)
    }

    @Test
    fun `updateNextArrowEnablingOnFragmentChange disabled as it is SCM && has skip logic but no answer`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val copiedVersion = questionUnderTest.copy(questionType = QuestionType.SingleChoice.value)
        every { skipLogicHandler.hasSkipLogic(copiedVersion.surveyQuestionGUID) } returns true
        viewModel.survey!!.questions!![2] = copiedVersion
        // call
        viewModel.updateNextArrowEnablingOnFragmentChange(2)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(false, t)
    }

    @Test
    fun `updateNextArrowEnablingOnFragmentChange disabled as SCM&&skip logic&&has answer but skipto ==-1`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val copiedVersion = questionUnderTest.copy(questionType = QuestionType.SingleChoice.value)
        val questionResponse = QuestionResponses(
            questionID = copiedVersion.surveyQuestionID,
            questionGUID = copiedVersion.surveyQuestionGUID,
            "rr"
        )
        viewModel.updateQuestionResponsesList(questionResponse)
        every { skipLogicHandler.hasSkipLogic(copiedVersion.surveyQuestionGUID) } returns true
        every { skipLogicHandler.getSkipToQuestionPosition(2, questionResponse) } returns -1
        viewModel.survey!!.questions!![2] = copiedVersion
        // call
        viewModel.updateNextArrowEnablingOnFragmentChange(2)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(false, t)
    }

    @Test
    fun `updateNextArrowEnablingOnFragmentChange enabled as SCM&&skip logic&&has answer&&skipto !=-1`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val copiedVersion = questionUnderTest.copy(questionType = QuestionType.SingleChoice.value)
        val questionResponse = QuestionResponses(
            questionID = copiedVersion.surveyQuestionID,
            questionGUID = copiedVersion.surveyQuestionGUID,
            "rr"
        )
        viewModel.updateQuestionResponsesList(questionResponse)
        every { skipLogicHandler.hasSkipLogic(copiedVersion.surveyQuestionGUID) } returns true
        every { skipLogicHandler.getSkipToQuestionPosition(2, questionResponse) } returns 3
        viewModel.survey!!.questions!![2] = copiedVersion
        // call
        viewModel.updateNextArrowEnablingOnFragmentChange(2)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(true, t)
    }

    @Test
    fun `updateSubmitAndNxtAfterAnswerChosen  nxt disabled as it is last Q`() {
        //setup data
        // call
        viewModel.updateSubmitAndNxtAfterAnswerChosen(viewModel.survey!!.questions!!.last().surveyQuestionGUID)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(false, t)
    }

    @Test
    fun `updateSubmitAndNxtAfterAnswerChosen  nxt enabled as it is not last Q and no skip logic`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        every { skipLogicHandler.hasSkipLogic(questionUnderTest.surveyQuestionGUID) } returns false
        // call
        viewModel.updateSubmitAndNxtAfterAnswerChosen(questionUnderTest.surveyQuestionGUID)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        assertEquals(true, t)
    }

    @Test
    fun `updateSubmitAndNxtAfterAnswerChosen  nxt disabled and submt shown as it is not last Q &&has skip logic but skip to ==-1`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        every { skipLogicHandler.hasSkipLogic(questionUnderTest.surveyQuestionGUID) } returns true
        every { skipLogicHandler.getSkipToQuestionPosition(2,null) } returns -1
        // call
        viewModel.updateSubmitAndNxtAfterAnswerChosen(questionUnderTest.surveyQuestionGUID)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        var t2: Pair<Int,Boolean>? = null
        viewModel.showSubmitButtonLD.observeOnce {
            t2 = it
        }
        assertEquals(false, t)
        assertEquals(true, t2!!.second)
        assertEquals(2, t2!!.first)
    }

    @Test
    fun `updateSubmitAndNxtAfterAnswerChosen  nxt enabled and submt hidden as it is not last Q &&has skip logic but skip to !=-1`() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        every { skipLogicHandler.hasSkipLogic(questionUnderTest.surveyQuestionGUID) } returns true
        every { skipLogicHandler.getSkipToQuestionPosition(2,null) } returns 4
        // call
        viewModel.updateSubmitAndNxtAfterAnswerChosen(questionUnderTest.surveyQuestionGUID)
        //verify
        var t: Boolean? = null
        viewModel.enableNextButtonLD.observeOnce {
            t = it
        }
        var t2: Pair<Int,Boolean>? = null
        viewModel.showSubmitButtonLD.observeOnce {
            t2 = it
        }
        assertEquals(true, t)
        assertEquals(false, t2!!.second)
        assertEquals(2, t2!!.first)
    }

    @Test
    fun `updateQuestionResponsesList  add `() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val questionResponse = QuestionResponses(
            questionGUID = questionUnderTest.surveyQuestionGUID,
            questionID = questionUnderTest.surveyQuestionID,
            textResponse = "txt"
        )
        // call
        viewModel.updateQuestionResponsesList(questionResponse)
        //verify
        assertEquals(viewModel.questionResponsesList.size, 1)
        assertEquals(viewModel.getQuestionResponseByPosition(2)?.textResponse, "txt")

    }
  @Test
    fun `updateQuestionResponsesList  update `() {
        //setup data
        val questionUnderTest = viewModel.survey!!.questions!![2]
        val questionResponse = QuestionResponses(
            questionGUID = questionUnderTest.surveyQuestionGUID,
            questionID = questionUnderTest.surveyQuestionID,
            textResponse = "txt"
        )
      val questionResponse2 = QuestionResponses(
            questionGUID = questionUnderTest.surveyQuestionGUID,
            questionID = questionUnderTest.surveyQuestionID,
            textResponse = "txt2"
        )
        // call
        viewModel.updateQuestionResponsesList(questionResponse)
        viewModel.updateQuestionResponsesList(questionResponse2)
        //verify
        assertEquals(viewModel.questionResponsesList.size, 1)
        assertEquals(viewModel.getQuestionResponseByPosition(2)?.textResponse, "txt2")

    }

    private fun getRequestBody(): RelevantWebSurveyBody {
        val surveyBody = RelevantWebSurveyBody()
        surveyBody.language = "lang"
        surveyBody.channel = "InApp"
        surveyBody.customerName = "saad"
        surveyBody.customerEmail = "ss@ss.com"
        surveyBody.customerPhone = "01234567890"
        surveyBody.dispositionCodes = listOf("complian Event")
        return surveyBody
    }

    private fun getQuestionList(): MutableList<Question> {
        val questionsList = mutableListOf<Question>()
        questionsList.add(
            Question(
                "abc", 1, "Multiple Choices",
                true, 2,
                listOf(
                    Choices("13e27ea8-0d3b-4d71-990c-a39c41815d8f", 30752, "1", "50eca54d", ""),
                    Choices("4f0bc9fb-66bb-4a47-8b21-a8d52bdaa089", 30751, "2", "fddb", ""),
                    Choices("a56c10c1-9541-4c05-8114-d4ccaf443f42", 30754, "3", "50eca54d", "")
                ),
                null
            )
        )

        questionsList.add(
            Question(
                "9c88051a-223b-4eac-a71c-c8e0d429e931", 2, "test input",
                true, 9,
                listOf(),
                null
            )
        )

        questionsList.add(
            Question(
                "96cb2bd9-4402-4438-90df-d5bad3c9213b", 3, "single Choices",
                true, 15,
                listOf(
                    Choices("fb1c62bd-7093-49b3-b6f1-7a6193c4a91c", 30750, "test", "50eca54d", ""),
                    Choices("d6225421-ecf3-4958-a48b-cc7770869129", 30756, "testt", "fddb", ""),
                    Choices(
                        "c7d2d1e4-43c3-46e5-9f8a-ec801c3528d0",
                        30759,
                        "testttt",
                        "50eca54d",
                        ""
                    )
                ),
                null
            )
        )

        questionsList.add(
            Question(
                "0122d1f3-f549-4e39-927e-b619f807df66", 4, "DatePicker",
                false, 5,
                listOf(),
                null
            )
        )
        questionsList.add(
            Question(
                "6a370f7b-075c-4fd8-b535-c6e26f6359fe", 36149, "Sliding Scale ",
                true, QuestionType.SlideQuestion.value,
                listOf(),
                null
            )
        )
        questionsList.add(
            Question(
                "acf6b89d-b8ae-4e96-81c4-b175f4753e13", 5, "Number",
                true, 10,
                listOf(),
                null
            )
        )

        questionsList.add(
            Question(
                "22f06b94-2292-4c13-abc4-634e1c44d472", 6, "Email",
                true, 11,
                listOf(),
                null
            )
        )

        questionsList.add(
            Question(
                "ae593422-7634-4e4a-afa8-566c867e4b5e", 7, "Phone",
                true, 12,
                listOf(),
                null
            )
        )
        questionsList.add(
            Question(
                "5dfeff9e-5af5-42dc-a2ed-60ee9912012b", 8, "URL Question",
                true, 14,
                listOf(),
                null
            )
        )
        return questionsList

    }

    private fun getSkipLogic(): MutableList<SkipLogic> {
        val skipList = mutableListOf<SkipLogic>()
        skipList.add(
            SkipLogic(
                id = 20286,
                questionGUID = "abc",
                qChoiceGUID = "13e27ea8-0d3b-4d71-990c-a39c41815d8f",
                minValue = 0,
                maxValue = 0,
                skipToQuestionGUID = "9c88051a-223b-4eac-a71c-c8e0d429e931"
            )
        )

        skipList.add(
            SkipLogic(
                id = 20288,
                questionGUID = "96cb2bd9-4402-4438-90df-d5bad3c9213b",
                qChoiceGUID = "fb1c62bd-7093-49b3-b6f1-7a6193c4a91c",
                minValue = 0,
                maxValue = 0,
                skipToQuestionGUID = "0122d1f3-f549-4e39-927e-b619f807df66"
            )
        )

        skipList.add(
            SkipLogic(
                id = 20289,
                questionGUID = "22f06b94-2292-4c13-abc4-634e1c44d472",
                qChoiceGUID = null,
                minValue = 0,
                maxValue = 0,
                skipToQuestionGUID = "5dfeff9e-5af5-42dc-a2ed-60ee9912012b"
            )
        )

        skipList.add(
            SkipLogic(
                id = 20289,
                questionGUID = "6a370f7b-075c-4fd8-b535-c6e26f6359fe",
                qChoiceGUID = null,
                minValue = 3,
                maxValue = 5,
                skipToQuestionGUID = "ae593422-7634-4e4a-afa8-566c867e4b5e"
            )
        )
        return skipList
    }

    private fun getMockedRepo(
        response: Resource<RelevantWebSurveyResponse>,
        requestBody: RelevantWebSurveyBody
    ): HiveSDKRepositoryImpl {
        return mockk {
            coEvery {
                getRelevantWebSurveyResource(
                    "saad",
                    "InApp2021",
                    requestBody
                )
            } returns response
        }
    }
}